import {
  Config, Singleton, Http, IDictionary, Loader, dayjs as moment, I18n, Metadata, IEventParam,
} from '@zeedhi/core';
import {
  LoadingService, Form, TextInput, DialogService,
} from '@zeedhi/common';
import { BottomSheet } from '@zeedhi/zd-bottom-sheet';
import 'dayjs/locale/pt';
import 'dayjs/locale/en';
import 'dayjs/locale/es';

@Singleton
export class LoginController {
  private utilities = Loader.getInstance('Utilities');

  public appController = Loader.getInstance('AppController');

  private notSettedServer = false;

  get logo() {
    return Config.env.product.logoUrl;
  }

  get backgroundImgUrl() {
    const url = Config.env.loginBackgroundImgUrl;
    return `url(${url})`;
  }

  get endPoint() {
    // return Config.env.loginEndPoint;
    if (Config.env.loginEndPoint === '') {
      const serverIp = localStorage.getItem('SERVER_URL');
      if (serverIp) {
        Config.env.loginEndPoint = process.env.NODE_ENV === 'production' ? `${serverIp}/backend_login/index.php`
          : `${serverIp}/modules/login/backend/service/index.php`;
      } else {
        Config.env.loginEndPoint = '';
      }
    }
    return Config.env.loginEndPoint;
  }

  private setLanguage() {
    const lang = this.utilities.getLanguage();
    I18n.changeLanguage(this.appController.metaLanguages[lang]);
    moment.locale(lang.substring(0, 2));
  }

  public openServerSheet() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('serverSheet');
    bottomSheet.isVisible = true;
    bottomSheet.isOpen = true;
  }

  public clickCloseSheetBtn() {
    const serverForm = Metadata.getInstance<Form>('formServerRepository');
    if (serverForm.validate() && this.notSettedServer === false) {
      const bottomSheet = Metadata.getInstance<BottomSheet>('serverSheet');
      bottomSheet.isVisible = false;
      bottomSheet.isOpen = false;
    }
  }

  public setServer() {
    const serverForm = Metadata.getInstance<Form>('formServerRepository');
    if (serverForm.validate()) {
      const ip = serverForm.value.ip?.replace(/\/$/, '') || '';
      const port = serverForm.value.port ? `:${serverForm.value.port}` : '';
      const path = serverForm.value.diretory.replace(/^\//g, '') || '';
      const endPoint = process.env.NODE_ENV === 'production' ? `${ip}${port}/${path}/backend/index.php`
        : `${ip}${port}/${path}/backend/service/index.php`;
      localStorage.setItem('CONFIG_ENDPOINT', Config.endPoint);
      Config.endPoint = endPoint;
      Config.env.loginEndPoint = process.env.NODE_ENV === 'production' ? `${ip}${port}/${path}/backend_login/index.php`
        : `${ip}${port}/${path}/modules/login/backend/service/index.php`;
      LoadingService.show(' ');
      Http.setBaseURL('');
      Http.get(`${Config.endPoint}/testConnection`).then((response: IDictionary) => {
        if (!response.data.error) {
          this.notSettedServer = false;
          Http.setBaseURL(Config.env.loginEndPoint);
          localStorage.setItem('SERVER_URL', `${ip}${port}/${path}`);
          localStorage.setItem('SERVER_IP', ip);
          localStorage.setItem('SERVER_PORT', port);
          localStorage.setItem('SERVER_DIRETORY', path);
          this.utilities.showAlert('successConnection', 'SUCCESS_CONNECTION', 'green');
          this.clickCloseSheetBtn();
        } else {
          const mensagem = response.data.error;
          console.warn(mensagem);
          this.utilities.showAlert('errorConnection', 'SERVER_CONNECTION_FAIL', 'error');
        }
      }).catch((error: string) => {
        console.warn(error);
        this.utilities.showAlert('alert', 'SERVER_CONNECTION_FAIL', 'error');
      }).then(() => {
        LoadingService.hide();
      });
    }
  }

  public getServer() {
    if (Config.endPoint === '') {
      const serverIp = localStorage.getItem('SERVER_URL');
      if (serverIp) {
        const endPoint = `${serverIp}/backend/service/index.php`;
        Config.endPoint = endPoint;
        Config.env.loginEndPoint = process.env.NODE_ENV === 'production' ? `${serverIp}/backend_login/index.php`
          : `${serverIp}/modules/login/backend/service/index.php`;
        // Http.setBaseURL(Config.endPoint);
      } else {
        Config.endPoint = '';
        Config.env.loginEndPoint = '';
        // Http.setBaseURL(Config.endPoint);
      }
    }
    Http.setBaseURL(Config.endPoint);
  }

  public getServerDiretory({ component }: IEventParam<TextInput>) {
    if (Config.endPoint !== '') {
      const url = new URL(Config.endPoint);
      component.value = url.pathname.replace('/backend/service/index.php', '');
      component.value = process.env.NODE_ENV === 'production'
        ? url.pathname.replace('/backend/index.php', '')
        : url.pathname.replace('/backend/service/index.php', '');
    } else component.value = localStorage.getItem('SERVER_DIRETORY') || '';
  }

  public getServerPort({ component }: IEventParam<TextInput>) {
    if (Config.endPoint !== '') {
      const url = new URL(Config.endPoint);
      component.value = url.port;
    } else component.value = localStorage.getItem('SERVER_PORT') || '';
  }

  public getServerIp({ component }: IEventParam<TextInput>) {
    if (Config.endPoint !== '') {
      const url = new URL(Config.endPoint);
      component.value = `${url.protocol}//${url.hostname}`;
    } else component.value = localStorage.getItem('SERVER_IP') || '';
  }

  public warnSetServer() {
    if (Config.endPoint === '') {
      DialogService.show(
        {
          name: 'paramServerRepository',
          type: 'warning',
          title: I18n.translate('RotinaEstoqueApp::ATTENTION'),
          text: I18n.translate('RotinaEstoqueApp::PARAM_SERVER'),
          buttons: [
            {
              name: 'dialogButton',
              label: I18n.translate('RotinaEstoqueApp::OK'),
              component: 'ZdButton',
              events: {
                click: () => {
                  DialogService.hide();
                  this.openServerSheet();
                  this.notSettedServer = true;
                },
              },
            },
          ],
        },
      );
    }
  }

  public fixDateFormat() {
    localStorage.setItem('DATEFORMAT', this.utilities.getFormatLocale());
    Config.displayFormat = this.utilities.getFormatLocale();
  }
}
